/**
 *****************************************************************************
 **
 **  File        : main.c
 **
 **  Abstract    : main function.
 **
 **  Functions   : main
 **
 **  Environment : Atollic TrueSTUDIO(R)
 **                STMicroelectronics STM32F4xx Standard Peripherals Library
 **
 **  Distribution: The file is distributed "as is", without any warranty
 **                of any kind.
 **
 **  (c)Copyright Atollic AB.
 **  You may use this file as-is or modify it according to the needs of your
 **  project. This file may only be built (assembled or compiled and linked)
 **  using the Atollic TrueSTUDIO(R) product. The use of this file together
 **  with other tools than Atollic TrueSTUDIO(R) is not permitted.
 **
 *****************************************************************************
 */

/* Includes */
#include "stdio.h"
#include "defines.h"
#include "tm_stm32f4_i2c.h"
#include "nunchuk.h"
#include "utilidades.h"

#include "stm32f4xx.h"
#include "stm32f4_discovery.h"
#include "tm_stm32f4_delay.h"
#include "tm_stm32f4_servo.h"

#include "lcd_hd44780.h"

//Slave address
#define NUNCHUK_ADDRESS 		0xA4		// 0xA4    Nunchuk Address
#define CLOCK_SPEED 			100000			// Speed for I2C
#define LOOP_DELAY 				50					// Delay to wait between iterations
#define SECONDS_TO_MAX			2
#define CYCLES 					2
#define MAX_JOY 				123					// Max value for Joysticks
#define MAX_ACC 				65					// Max value for Roll and Pitch

#define INIT_MESSAGE_BLINK 3
#define WAIT_SERVO 0

#define SHOULDER_HMIN 0				// 0 is right position
#define SHOULDER_HMAX 180			// 180 is left position

#define SHOULDER_VMIN 5
#define SHOULDER_VMAX 120

#define CUBIT_MIN 0
#define CUBIT_MAX 180

#define WRIST_VMIN 65
#define WRIST_VMAX 175

#define WRIST_RMIN 0
#define WRIST_RMAX 180

#define CLAW_MIN 0
#define CLAW_MAX 90

#define MIN_VWRIST 10
#define MIN_RWRIST 10

/* Private macro */
/* Private variables */
TM_SERVO_t ShoulderHorizontal, ShoulderVertical, Cubit,
		WristVertical, WristRotation, Claw;

int SHOULDER_HSTEP = 0;
int SHOULDER_VSTEP = 0;
int CUBIT_STEP  = 10;
int WRIST_VSTEP = 5;
int WRIST_RSTEP = 5;
int CLAW_STEP = 1;					// Step for opening and closing the claw

int shoulderHPosition = 90;
int shoulderVPosition = 45;
int clawAperture = 45;
int cubitPosition = 90;
int wristRPosition = 90;
int wristVPosition = 180;
int encendido = 0;
/* Private function prototypes */

/* Private functions */

/**
 * Method to control if C + Z buttoms have been pressed to Start System.
 */
int powerOnControl(int cPressed, int zPressed) {
	int resultado = 0;
	if (encendido == 0 && cPressed == 1 && zPressed == 1) {
		nunchuk_SendZero();
		Delay(2000);
		nunchuk_Update();
		int cPressed2 = nunchuk_GetC();
		int zPressed2 = nunchuk_GetZ();
		if (cPressed2 == 1 && zPressed2 == 1) {
			resultado = 1;
			encendido = 1;
			int i = 0;
			for (i = 0; i < INIT_MESSAGE_BLINK; i++) {
				lcd_cls();
				Delayms(1000);
				lcd_strxy("Initializing ",0,0);

				lcd_strxy("System...",0,1);
				Delayms(1000);
			}
			lcd_cls();
			lcd_PrintLabels();
			/* Set initial position */
			TM_SERVO_SetDegrees(&ShoulderHorizontal, shoulderHPosition);	// 90 centers arm in the center.
			TM_SERVO_SetDegrees(&ShoulderVertical, shoulderVPosition);		// 45 centers arm in the center of elevation.

			TM_SERVO_SetDegrees(&Cubit, cubitPosition);						// 90 puts arm in the middle of lenght

			TM_SERVO_SetDegrees(&WristRotation, wristRPosition);			// 90 centers arm in the center of rotation
			TM_SERVO_SetDegrees(&WristVertical, wristVPosition);			// 180 puts the claw straight

			TM_SERVO_SetDegrees(&Claw, clawAperture);						// 45 puts claw aperture in the middle of total aperture
		}
	}
	return resultado;
}

/**
 * Method to declarate and initialize arm servos.
 */
void initServos() {
	TM_SERVO_Init(&ShoulderHorizontal, TIM5, TM_PWM_Channel_1, TM_PWM_PinsPack_1);	//PA0  Green -> Shoulder Horizontal
	TM_SERVO_Init(&ShoulderVertical, TIM5, TM_PWM_Channel_2, TM_PWM_PinsPack_1); 	//PA1  Yellow -> Shoulder Vertical

	TM_SERVO_Init(&Cubit, TIM4, TM_PWM_Channel_1, TM_PWM_PinsPack_2); 				//PD12  Orange -> Cubit
	TM_SERVO_Init(&WristVertical, TIM4, TM_PWM_Channel_2, TM_PWM_PinsPack_2); 		//PD13  Red -> Wrist Vertical

	//TM_SERVO_Init(&WristRotation, TIM4, TM_PWM_Channel_3, TM_PWM_PinsPack_2); 		//PD14  Grey -> Wrist Rotation
	//TM_SERVO_Init(&Claw, TIM4, TM_PWM_Channel_4, TM_PWM_PinsPack_2); 				//PD15  Black -> Claw

	TM_SERVO_Init(&WristRotation, TIM8, TM_PWM_Channel_1, TM_PWM_PinsPack_1);		//PC6	Grey -> Wrist Rotation
	TM_SERVO_Init(&Claw, TIM8, TM_PWM_Channel_2, TM_PWM_PinsPack_1); 				//PC7	Black -> Claw
}

/**
 * Method to update horizontal position of shoulder servo.
 */
void updateShoulderHorizontal(int value) {
//	int degrees = map(value, -MAX_JOY, MAX_JOY, 180, 0);
//	TM_SERVO_SetDegrees(&ShoulderHorizontal, degrees);
	int ant = shoulderHPosition;
	int degrees = map(value, -MAX_JOY, MAX_JOY, SHOULDER_HSTEP, -SHOULDER_HSTEP);
	shoulderHPosition += degrees;
	shoulderHPosition = normalizeValue(shoulderHPosition, SHOULDER_HMIN, SHOULDER_HMAX);
	if (shoulderHPosition != ant) {
		TM_SERVO_SetDegrees(&ShoulderHorizontal, shoulderHPosition);
		//Delayms(WAIT_SERVO);
	}
}

/**
 * Method to update vertical position of shoulder servo.
 */
void updateShoulderVertical(int value, int c, int z) {
	if (c == 1 && z == 1) {
//		int degrees = map(value, -MAX_ACC, MAX_ACC, 180, 0);
//		TM_SERVO_SetDegrees(&ShoulderVertical, degrees);
		int ant = shoulderVPosition;

		int degrees = map(value, -MAX_ACC, MAX_ACC, SHOULDER_VSTEP, -SHOULDER_VSTEP);
		shoulderVPosition += degrees;
		shoulderVPosition = normalizeValue(shoulderVPosition, SHOULDER_VMIN, SHOULDER_VMAX);
		if (shoulderVPosition != ant) {
			TM_SERVO_SetDegrees(&ShoulderVertical, shoulderVPosition);
			//Delayms(WAIT_SERVO);
		}
	}
}

/**
 * Method to update position of cubit servo.
 */
void updateCubit(int value) {
//	int degrees = map(value, -MAX_JOY, MAX_JOY, 180, 0);
//	TM_SERVO_SetDegrees(&Cubit, degrees);
	int ant = cubitPosition;
	int degrees = map(value, -MAX_JOY, MAX_JOY, -CUBIT_STEP, CUBIT_STEP);
	cubitPosition += degrees;
	cubitPosition = normalizeValue(cubitPosition, CUBIT_MIN, CUBIT_MAX);
	if (cubitPosition != ant) {
		TM_SERVO_SetDegrees(&Cubit, cubitPosition);
		//Delayms(WAIT_SERVO);
	}
}

/**
 * Method to update vertical position of wrist servo
 */
void updateWristVertical(int value, int c, int z) {

	if (c == 0 || z == 0) {
		int ant = wristVPosition;
//		int degrees = map(value, -MAX_ACC, MAX_ACC, 180, 0);
//		TM_SERVO_SetDegrees(&WristVertical, degrees);
		int degrees = 0;
		//map(value, -MAX_ACC, MAX_ACC, WRIST_VSTEP, -WRIST_VSTEP);
		if (abs(value) < MIN_VWRIST) {
			degrees = 0;
		} else {
			degrees = map(abs(value), MIN_VWRIST, MAX_ACC, 0, WRIST_VSTEP);
		}
		if (value < 0) {
			degrees = -degrees;
		}

		wristVPosition = wristVPosition + degrees;
		wristVPosition = normalizeValue(wristVPosition, WRIST_VMIN, WRIST_VMAX);
		if (wristVPosition != ant) {
			TM_SERVO_SetDegrees(&WristVertical, wristVPosition);
			//Delayms(WAIT_SERVO);
		}
	}
}

/**
 * Method to update rotation position of wrist servo
 */
void updateWristRotation(int value) {

	//int degrees = map(value, -MAX_ACC, MAX_ACC, 0, 180);
	//TM_SERVO_SetDegrees(&WristRotation, degrees);
	int ant = wristRPosition;
	int degrees = 0;
	//map(value, -MAX_ACC, MAX_ACC, -WRIST_RSTEP, WRIST_RSTEP);

	if (abs(value) < MIN_RWRIST) {
		degrees = 0;
	} else {
		degrees = map(abs(value), MIN_RWRIST, MAX_ACC, 0, WRIST_RSTEP);
	}
	if (value < 0) {
		degrees = -degrees;
	}
	wristRPosition = wristRPosition + degrees;
	wristRPosition = normalizeValue(wristRPosition, WRIST_RMIN, WRIST_RMAX);
	if (wristRPosition != ant) {
		TM_SERVO_SetDegrees(&WristRotation, wristRPosition);
		//Delayms(WAIT_SERVO);
	}
}

/**
 * Method to update apperture of claw.
 */
void updateClaw(int cPressed, int zPressed) {

	 if ((cPressed == 0 || zPressed == 0) && (cPressed == 1 || zPressed == 1)) {
		 int ant = clawAperture;
		 if (cPressed == 1) {
			 clawAperture += CLAW_STEP;
		 } else if (zPressed == 1) {
			 clawAperture -= CLAW_STEP;
		 }
		 clawAperture = normalizeValue(clawAperture, CLAW_MIN, CLAW_MAX);
		 if (clawAperture != ant) {
			 TM_SERVO_SetDegrees(&Claw, clawAperture);
			 //Delayms(WAIT_SERVO);
		 }
	 }
}

/**
 * Method to update servo position
 */
void updateArm(int joystickX, int joystickY, int roll, int pitch, int cPressed,	int zPressed) {
	updateShoulderHorizontal(joystickX);
	updateShoulderVertical(pitch, cPressed, zPressed);
	updateCubit(joystickY);
	updateWristVertical(pitch, cPressed, zPressed);
	updateWristRotation(roll);
	updateClaw(cPressed, zPressed);
}

/**
 * Method to print labels on LCD
 */
void lcd_PrintLabels() {

	lcd_strxy("X:",0,0);
	lcd_strxy("Y:",7,0);
	lcd_strxy("C",14,0);

	lcd_strxy("R:",0,1);
	lcd_strxy("P:",7,1);
	lcd_strxy("Z",14,1);
}

/**
 * Method to print nunchuk values on LCD
 */
void lcd_ShowNunchukValues(int joystickX, int joystickY, int roll, int pitch,
		int cPressed, int zPressed) {

	lcd_strxy("    ",2,0);
	lcd_intxy(joystickX,2,0);
	lcd_strxy("    ",9,0);
	lcd_intxy(joystickY,9,0);
	lcd_intxy(cPressed,15,0);

	lcd_strxy("    ",2,1);
	lcd_intxy(roll,2,1);
	lcd_strxy("    ",9,1);
	lcd_intxy(pitch,9,1);
	lcd_intxy(zPressed,15,1);
}

/**
 * Method to calculate servos step.
 */
void initSteps() {
	int iterationSecond = SECONDS_TO_MAX * (1000/LOOP_DELAY);
	SHOULDER_HSTEP = SHOULDER_HMAX/(2 * iterationSecond);
	SHOULDER_VSTEP = SHOULDER_VMAX/(2 * iterationSecond);;
	CUBIT_STEP  = CUBIT_MAX/(2 * iterationSecond);;
	WRIST_VSTEP = WRIST_VMAX/(2 * iterationSecond);
	WRIST_RSTEP = WRIST_RMAX/(2 * iterationSecond);;
	CLAW_STEP = CLAW_MAX/(2 * iterationSecond);;
}

/**
 **===========================================================================
 **
 **  Abstract: main program
 **
 **===========================================================================
 */
int main(void) {
	//Initialize system
	SystemInit();

	/* Initialize delay */
	TM_DELAY_Init();

	/* Initialize I2C for write/read to/from Nunchuk */
	nunchuk_Init(I2C1, NUNCHUK_ADDRESS, TM_I2C_PinsPack_1, CLOCK_SPEED); // SCL PB6 and SDA PB7

	/* Initialize servos */
	initServos();

	/* Initialize LCD */
	lcd_init();

	initSteps();

	/* Print Labels on LCD */
	//lcd_PrintLabels();
	lcd_strxy("System OFF Press",0,0);
	lcd_strxy("C+Z 2s to Start",0,1);

	while (1) {		// Infinite Loop
		/* Update Nunchuk values */
		if (nunchuk_Update() == 1) {

			/* Read each values from Nunchuk */
			int joystickX = nunchuk_GetJoystickX();
			int joystickY = nunchuk_GetJoystickY();
			int roll = nunchuk_GetRoll();
			int pitch = nunchuk_GetPitch();
			int cPressed = nunchuk_GetC();
			int zPressed = nunchuk_GetZ();

			if (!encendido) {
				powerOnControl(cPressed, zPressed);
			} else {
				/* Update positions of all servos in arm */
				updateArm(joystickX, joystickY, roll, pitch, cPressed, zPressed);

				/* Show Nunchuk values on LCD */
				lcd_ShowNunchukValues(joystickX, joystickY, roll, pitch, cPressed, zPressed);
			}
		}

		/* Prepare Nunchuk for new read */
		nunchuk_SendZero();

		/* Wait for DELAY milliseconds for a new execution */
		//delay(2500000);
		Delayms(LOOP_DELAY);
	}
}

/*
 * Callback used by stm32f4_discovery_audio_codec.c.
 * Refer to stm32f4_discovery_audio_codec.h for more info.
 */
void EVAL_AUDIO_TransferComplete_CallBack(uint32_t pBuffer, uint32_t Size) {
	/* TODO, implement your code here */
	return;
}

/*
 * Callback used by stm324xg_eval_audio_codec.c.
 * Refer to stm324xg_eval_audio_codec.h for more info.
 */
uint16_t EVAL_AUDIO_GetSampleCallBack(void) {
	/* TODO, implement your code here */
	return -1;
}
